# =============================================================================
# @file    test_is_person.py
# @brief   Py.test cases for parts of name_utils.py
# @created 2022-12-23
# @license Please see the file named LICENSE in the project directory
# @website https://github.com/caltechlibrary/iga
# =============================================================================

from collections import namedtuple
from iga.name_utils import (
    split_name,
    is_person,
)

# Some of these names came from the following gist accessed on 2023-01-17:
# https://gist.github.com/paulmillr/2657075/a31455729440672467ada20ac10452d74a871e54

# Some real-life examples that we can't handle.
#
# * Ingy döt Net -- someone being cute, but it's not a real name
#
# * Wynn Netherland -- spaCy thinks it's a geopolitical entity, which IMHO is
# actually reasonable (I wouldn't be sure it's a person without Googling).
#
# * Rico Sta. Cruz -- "Sta. Cruz" is probably Santa Cruz, but neither PP nor
# nameparser figure it out and instead they think "Sta" is a middle name.
#
# * hitode909, lijing00333 -- these are login handles, not human names, so
# it's perfectly reasonable we can't classify them as person or company.
#
# * Company names Ovid, A.G. Edwards, H.B. Fuller are human names, so these are
# ambiguous cases. Currently we fail on them and I don't know what to do.
#
# The following examples are businesses and I can't figure out what to do. PP
# *could* tag them as companies, but PP also erroneously tags some human names
# as company, so the only way to avoid incorrectly tagging human names as
# companies is to *always* ignore PP's tag if it comes out as company
# -- which means we always fail on these:
#
# * Account for Github research
# * Practical Arduino - The Book
# * Sony Xperia Developer World

PEOPLE = [
    "Martin Luther King, Jr.",
    "Yihui Xie",
    "José Valim",
    "Kevin Sawicki",
    "Jordan Sissel",
    "Alon Zakai",
    "Erik Michaels-Ober",
    "Shougo",
    "Edward A. Kmett",
    "Sven Fuchs",
    "Ryan Bigg",
    "Nathan Rajlich",
    "Dominic Tarr",
    "David Fowler",
    "Dr Nic Williams",
    "Michael Snoyman",
    "Kohsuke Kawaguchi",
    "Yichun Zhang",
    "Paul Miller",
    "Ayende Rahien",
    "Tokuhiro Matsuno",
    "Aaron Heckmann",
    "Kenneth Reitz",
    "Lukas Kahwe Smith",
    "William Durand",
    "lifesinger",
    "Benjamin Eberlei",
    "Hsiaoming Yang",
    "Fedor Indutny",
    "Igor Wiedler",
    "Jordi Boggiano",
    "Daniel Greenfeld",
    "Zeno Rocha",
    "Irakli Gozalishvili",
    "Ask Solem Hoel",
    "Cristi Burcă",
    "Durran Jordan",
    "Rico Sta. Cruz",
    "Linus Torvalds",
    "Joshua Timberman",
    "Benjamin Arthur Lupton",
    "mattn",
    "Philip (flip) Kromer",
    "Derick Bailey",
    "Will Bond",
    "Anil Madhavapeddy",
    "Yusuke Suzuki",
    "Scott González",
    "Lloyd Hilaiel",
    "Wes McKinney",
    "Konstantin Kudryashov",
    "Demis Bellot",
    "Eric Meyer",
    "Mattt Thompson",
    "Dustin Sallings",
    "John Wiegley",
    "Maxwell Salzberg",
    "Magnar Sveen",
    "Dale  Harvey",
    "Benoit Chesneau",
    "David Grudl",
    "Phil Hagelberg",
    "James Burke",
    "Samuel Clay",
    "Jeremy D. Miller",
    "Addy Osmani",
    "Henri Bergius",
    "Adam Vandenberg",
    "Steve Losh",
    "Luke Daley",
    "Jared Hanson",
    "Philip Arndt",
    "Rick Waldron",
    "Mark Story",
    "John MacFarlane",
    "Jeremy Ashkenas",
    "David Dollar",
    "Mike Bostock",
    "R. Tyler Croy",
    "Mark Otto",
    "Wesley Beary",
    "Ricardo Signes",
    "Gregory Brown",
    "John-David Dalton",
    "Paul Betts",
    "Paul Reioux",
    "Tony Arcieri",
    "Kana Natsuno",
    "Ben Alman",
    "Astro",
    "Jeff Forcier",
    "Sindre Sorhus",
    "Marijn Haverbeke",
    "Richard Huang",
    "Tim Caswell",
    "Joshua Peek",
    "Mitchell Hashimoto",
    "James Coglan",
    "Rafael Mendonça França",
    "Michael Aufreiter",
    "Damien Elmes",
    "Jake Wharton",
    "Yuvi Panda",
    "Yurii Rashkovskii",
    "Andrews Medina",
    "Konstantin Haase",
    "Jörn Zaefferer",
    "Daniel Shiffman",
    "Josh Abernathy",
    "Anthony Grimes",
    "Randall Degges",
    "Eloy Durán",
    "Travis Swicegood",
    "Amos Wenger",
    "Remy Sharp",
    "Nathan Marz",
    "Jeremy Kahn",
    "Bryan O'Sullivan",
    "Nicolas Perriault",
    "Peter Steinberger",
    "Ben Noordhuis",
    "Shay Banon",
    "Guillermo Rauch",
    "Christophe Coevoet",
    "Jeff Casimir",
    "Mickael Daniel",
    "Carl Meyer",
    "Mr.doob",
    "hadley wickham",
    "Antonin Hildebrand",
    "Jasper Van der Jeugt",
    "Rebecca Murphey",
    "Christian Amor Kvalheim",
    "Evan Coury",
    "Jakub Vrána",
    "Mark McGranaghan",
    "Steve Klabnik",
    "Ricardo Quesada",
#    "Ingy döt Net",
    "lestrrat",
    "Aaron Patterson",
    "Aaron Straup Cope",
    "Thomas Davis",
    "Charles Oliver Nutter",
    "Kris Kowal",
    "Douglas Campos",
    "Aslak Hellesøy",
    "Yukihiro &quot;Matz&quot; Matsumoto",
    "Paul Irish",
    "Jose Diaz-Gonzalez",
    "Tim Branyen",
    "Ryan Weaver",
    "Derick Rethans",
    "Shawn M Moore",
    "Christian Johansen",
    "Charlie Robbins",
    "Scott Jehl",
    "Phil Sturgeon",
    "José Lorenzo Rodríguez",
    "Xinchen Hui",
    "Tomas Doran",
    "Peter Wagenet",
    "Josh Suereth",
    "Jackson Tian",
    "Lance Pollard",
    "Sebastian Bergmann",
    "Carlos Antonio da Silva",
    "Alessandro Franceschi",
    "Xueqiao Xu",
    "Kyle McDonald",
    "Jeff Verkoeyen",
    "Nadim Kobeissi",
    "Johannes",
    "James George",
    "Sam Soffes",
    "Jannis Leidel",
    "John Mettraux",
    "Mark Harrah",
    "Dan Kubb",
    "Olivier Grisel",
    "Tomás Pollak",
    "Piotr Sarnacki",
    "Felix Geisendörfer",
    "Pedro Teixeira",
    "saber",
    "Sean Cribbs",
    "Ariya Hidayat",
    "Naitik Shah",
    "Marcus Westin",
    "Coda Hale",
    "James Ward",
    "Mislav Marohnić",
    "Álvaro Justen",
    "Brian Noguchi",
#    "Wynn Netherland",
    "Koushik Dutta",
    "Oliver Drobnik",
    "Richard Schneeman",
    "Salvatore Sanfilippo",
    "fat-kun",
    "Christopher Jeffrey",
    "SHIBATA Hiroshi",
    "Miller Medeiros",
    "David DeSandro",
    "Piotr Solnica",
    "Bernardo Heynemann",
    "Matthew Eernisse",
    "Lim Chee Aun",
    "John Myles White",
    "James Tauber",
    "arturo",
    "Andy Lester",
    "Andrey “A.I” Sitnik",
    "Nuno Job",
    "rick",
    "Mathias Meyer",
    "Yi-Ting Cheng",
    "Eric Holscher",
    "Ricardo Cerqueira",
    "Vojta Jina",
    "Andrew Nacin",
    "Jon Leighton",
    "John Nunemaker",
    "Nicholas Semendyaev",
    "Allan Odgaard",
    "Michael Ficarra",
    "Maciej Małecki",
    "Ryan Davis",
    "Jaisen Mathai",
    "Sergey Chikuyonok",
    "Vicent Martí",
    "FURUHASHI Sadayuki",
    "James Reeves",
    "Arnout Kazemier",
    "Dean Sofer",
    "Daniel Hillenbrand",
    "Joshua Holbrook",
    "Charles Leifer",
    "Mike Perham",
    "David Walsh",
    "Dave Reisner",
    "Thomas Reynolds",
    "Phus Lu",
    "Javier Jiménez",
    "Fuji, Goro",
    "Justin Hileman",
    "Vincent Driessen",
    "Jason Davies",
    "Caolan McMahon",
    "Igor Minar",
    "Jonathan &quot;Duke&quot; Leto",
    "Zach Holman",
    "Patrick Debois",
    "AlteredQualia",
    "Hongli Lai",
    "Matthew McCullough",
    "Luke Redpath",
    "Fogus",
    "Nathan Weizenbaum",
    "Xavier Noria",
    "Nathan Hamblen",
    "Jed Schmidt",
    "Stevan Little",
    "Chris Done",
    "Bozhidar Batsov",
    "Stefan Kanev",
    "Craig Campbell",
#    "antimatter15",
    "Ian Bicking",
    "Dennis Reimann",
    "Miguel de Icaza",
    "Mathias Bynens",
    "Ward Cunningham",
    "Tim Pope",
    "Nicolas Gramlich",
    "Sam Pullara",
    "Aldo Cortesi",
    "Tatsuhiko Miyagawa",
    "Juan Lopes",
    "Evan Phoenix",
    "Kang-min Liu",
    "Jan Lehnardt",
    "Rob Allen",
    "thinca",
    "Jonas Nicklas",
    "R.I.Pienaar",
    "Steve Kondik",
    "Jim Weirich",
    "Jason Lee",
    "jerpelea",
    "Max Lapshin",
    "Alex MacCaw",
    "Yehuda Katz",
    "Grigory Bakunov",
    "Bert Belder",
    "Pedro Melo",
    "Jonathan Wight",
    "James Smith",
    "Daniel Sperl",
    "Harper Reed",
    "Brad Fitzpatrick",
    "Johan Tibell",
    "Assaf Arkin",
    "Christoph Pojer",
    "Cal Henderson",
    "Eric Bidelman",
    "David Keegan",
    "Akira Matsuda",
    "Divya Manian",
    "Mihai Bazon",
    "Alex Young",
    "Bryan Veloso",
    "Chris O'Hara",
    "Harry Roberts",
    "Eric Barnes",
    "Rob &quot;Hurricane&quot; Ashton",
    "Tobias Schneider",
    "astaxie",
    "Nils Adermann",
    "Alexander Makarov",
    "Young Hahn",
    "Artur Adib",
    "Mahdi Yusuf",
    "Mark Nottingham",
    "Paul Rouget",
    "Gabriel Falcão",
    "Brett Terpstra",
    "Ralph Schindler",
    "Hakim El Hattab",
    "Brian Rosner",
    "Dayle Rees",
    "Nick Sieger",
    "John Polacek",
    "dennis zhuang",
    "Magnus Woldrich",
    "Alexandre Gomes Gaigalas",
    "Allan Jardine",
    "Karel Minarik",
    "Bernhard Schussek",
    "Olivier Poitrey",
    "Matthias Tretter",
    "Francois Zaninotto",
    "Kentaro Kuribayashi",
    "Alexis Sellier",
    "Nicolas Gallagher",
    "Tualatrix Chou",
    "Evan Martin",
    "Jeffrey Zhao",
    "Lyric Wai",
    "Chris Banes",
    "Tim Sharpe",
    "Aaron Swartz",
    "Bodil Stokke",
    "Xavier Shay",
    "Nick Quaranto",
    "Carbo Kuo",
    "Tekkub",
    "Alexander Artemenko",
    "Rob Hudson",
    "Gosuke Miyashita",
    "David Chelimsky",
    "André Arko",
    "Keiji, Yoshimi",
    "Jacob Appelbaum",
    "Anton Kovalyov",
    "Corey Donohoe",
    "Irene Ros",
    "Gina Trapani",
    "Joshua Clayton",
    "Michael G. Schwern",
    "Chris Anderson",
    "Alex Sexton",
    "Alex Gaynor",
    "Ryan Grove",
    "Andy Dawson",
    "Paul Querna",
    "Ivan Sagalaev",
    "Juriy Zaytsev",
    "Fred Wu",
    "Shota Fukumori",
    "Jeff Lindsay",
    "Josh Lockhart",
    "Mary Rose Cook",
    "Pat Allan",
    "Memo Akten",
    "Matt Aimonetti",
    "Zach Carter",
    "Valerio Proietti",
    "Tomasz Figa",
    "youpy",
    "Davies Lliu",
    "Ethan Schoonover",
    "cho45",
    "Feross Aboukhadijeh",
    "Adán Miguel Sánchez Albert",
    "TAKANO Mitsuhiro",
    "Qiang Xue",
#    "Weibin Yao(姚伟斌)",
    "Max Goodman",
    "Idan Gazit",
    "Endy Muhardin",
    "Shaun McCormick",
    "Mikito Takada",
    "ara.t.howard",
    "Gareth Rushgrove",
    "adam j. sontag",
    "Jon Rohan",
    "Bartek Szopka",
    "Santiago Pastorino",
    "Mikeal Rogers",
    "Nate Abele",
    "Bulat Shakirzyanov",
    "David Heinemeier Hansson",
    "Armin Ronacher",
    "Max Ogden",
    "Ryan Bates",
    "Andreas Gal",
    "Jamie Rumbelow",
    "Corey Haines",
    "Sebastian Tschan",
    "Caleb Troughton",
    "Theodore Watson",
    "Jerome Gravel-Niquet",
    "James Turnbull",
    "Yuichiro MASUI",
    "Guillaume Bort",
    "Jeremy Bush",
#    "hitode909",
    "Magnus Holm",
    "Arun Agrawal",
    "Miško Hevery",
#    "云风",
    "James Dennis",
    "Guilherme Blanco",
    "Luciano Ramalho",
    "Jeremy Kemper",
    "Yoshihiro Sugi",
    "Dan Croak",
    "Alex Ott",
    "Lea Verou",
    "Elijah Insua",
    "Kris Zyp",
    "finscn",
    "Jonas Bonér",
    "Mat Marquis",
    "Keith Rarick",
    "Mohamed Mansour",
    "James Golick",
    "Stuart Sierra",
    "James Padolsey",
    "Michel Martens",
    "Dexter.Yy",
    "Julia West",
    "Scott Chacon",
    "Shuo Chen",
    "Ilya Grigorik",
    "Mark Murphy",
    "Kit Cambridge",
    "Lourens Naudé",
    "Brock Whitten",
    "Limor &quot;Ladyada&quot; Fried",
    "Josh Nichols",
    "brian d foy",
    "Ben Edmunds",
    "Jonathan H. Wage",
    "jugyo",
    "Gianni Chiappetta",
    "Henrique Bastos",
    "Martin Grenfell",
    "Brian LeRoux",
    "Brandon Keepers",
    "James Chen",
    "Brad Larson",
    "Heather Arthur",
    "Wayne E. Seguin",
    "Mark Jaquith",
    "Karl Seguin",
    "Nando Vieira",
    "John Resig",
    "Rob Fuller",
    "Sam Vermette",
    "Trevor Turk",
    "simurai",
    "Eric Butler",
    "Kevin Ballard",
    "Rasmus Andersson",
    "Kazuho Oku",
    "Daisuke Murase",
    "Chris Williams",
    "motemen",
    "Daniel Kerr",
    "Aman Gupta",
    "Darcy Laycock",
    "Robbie Hanson",
    "Swaroop C H",
    "Ryan Petrich",
    "Zachary Voase",
    "Mike Chambers",
    "Nick Lockwood",
    "Kris Wallsmith",
    "Jack Danger Canty",
    "Felix Gnass",
    "Chris Eppstein",
    "Moxie Marlinspike",
    "Thomas Fuchs",
    "Kyle Simpson",
    "Michael Bleigh",
    "Phil Haack",
    "Drew Neil",
    "Trek Glowacki",
    "Max Howell",
    "Cody Lindley",
    "Dave Gamache",
    "Jamie Wilkinson",
    "Joe Hewitt",
    "Peter Seibel",
    "Steven Sanderson",
    "Damian Edwards",
    "Daniel Shaw",
    "Alvaro Videla",
    "Mugunth Kumar",
    "Jacob Kaplan-Moss",
    "Gary Bernhardt",
    "Scott Schiller",
    "Peteris Krumins",
    "honcheng",
    "Paul Kinlan",
    "Sam Stephenson",
    "Shaun Smith",
    "Victor Coulon",
    "Tianyi Cui",
    "Yusuke Yamamoto",
    "Ivo Wetzel",
    "Paul Lewis",
    "Michael Hartl",
    "Yoshimasa Niwa",
    "komagata",
    "Daniel Lindsley",
    "Brandon Mathis",
    "Brad Frost",
    "Dustin Diaz",
    "Charlie McConnell",
    "James Bennett",
    "Florian Ragwitz",
    "Michael Fellinger",
    "Paolo Fragomeni",
    "Jonathan 'Wolf' Rentzsch",
    "Abraham Williams",
    "Pavel",
    "Nick Fitzgerald",
    "Guillaume Laforge",
    "James Edward Gray II",
    "Robby Russell",
    "Scott Hanselman",
    "Devin Price",
    "Brian Lopez",
    "Qiangning Hong",
    "Kakutani Shintaro",
    "Kevin van Zonneveld",
    "Albert Pham",
    "Christian Neukirchen",
    "Chris Granger",
    "Mike Alsup",
    "Rasmus Lerdorf",
    "Harry Marr",
    "Luís Almeida",
    "Uli Kusterer",
    "Avdi Grimm",
    "Ryan Tomayko",
    "Shaofei Cheng",
    "Juan Basso",
    "Liam McLoughlin",
    "Jon Maddox",
    "Justin Palmer",
    "Tom McFarlin",
    "Justin Abrahms",
    "Thiago Belem",
    "Martian Z",
    "Cédric Luthi",
    "YAMAGUCHI EIKICHI",
    "James Tucker",
    "Kazuhiro Osawa",
    "Christian Heilmann",
    "Justin Windle",
    "Tom Dale",
    "Marc-André Cournoyer",
    "Satoshi Nakagawa",
    "Nathan Smith",
    "Ben Firshman",
    "Steve Francia",
    "Boris Smus",
    "Nicolás Sanguinetti",
    "Richard Crowley",
    "Aaron Quint",
    "Jeff Gilfelt",
    "Shi Chuan",
    "Tomohito Ozaki",
    "chromatic",
    "Wen-Tien Chang",
    "Graham Weldon",
    "Ernie Miller",
    "Blake Mizerany",
    "Pádraic Brady",
#    "lijing00333",
    "Larry Wall",
    "Brandon Jones",
    "David Nolen",
    "James Tang",
    "Leevi Graham",
    "Ben Schwarz",
    "Rei",
    "Evan Miller",
    "Takuto Wada",
    "Giles",
    "Michael Fields",
    "Eric Davis",
    "John-Paul Bader",
    "Pamela Fox",
    "Ed Finkler",
    "Ryan Seddon",
    "Jean Carlo Nascimento",
    "August &quot;Gus&quot; Mueller",
    "Edwin Chen",
    "Simon Rozet",
    "Fabio Kung",
    "Christiano Milfont",
    "Anthony Ettinger",
    "Carl Lerche",
    "Robey Pointer",
    "Dustin L. Howett",
    "Boris Moore",
    "John Barnette",
    "nicolas garcia belmonte",
    "Joshua Hull",
    "Peter Alfonso",
    "Timothee &quot;TTimo&quot; Besset",
    "Dmitri",
    "Dan Horrigan",
    "Kevin Whinnery",
    "Egor Homakov",
    "Bryan Helmkamp",
    "Ciaran Jessup",
    "Reg Braithwaite",
    "Christophe Coenraets",
    "Dinner Bone",
    "Guilherme Silveira",
    "Mitchell Simoens",
    "Zed A. Shaw",
    "Evan Wallace",
    "Nicholas C. Zakas",
    "Douglas Crockford",
    "Loiane Groner",
    "Jack Moffitt",
    "Tim Lucas",
    "Seth Fitzsimmons",
    "Jeff Haynie",
    "Yuichi Tateno",
    "Gabriel Handford",
    "朱建刚",
    "Dan Eden",
    "Scott Taylor",
    "Karlisson Bezerra",
    "Matt Biddulph",
    "Tom Preston-Werner",
    "Chris Wanstrath",
    "Paul Tarjan",
    "Justin French",
    "Johan Nilsson",
    "Adam Shanks",
    "Wang, Xiaozhe",
    "Tobias Lütke",
    "Stuart Halloway",
    "Kyle Neath",
    "Kishikawa Katsumi",
    "Douglas Neiner",
    "Amir Salihefendic",
    "Yusuke Wada",
    "Aral Balkan",
    "Tyler Hall",
    "Jeremy McAnally",
    "Chad Scira",
    "Marcus Ramberg",
    "Woody Gilk",
    "James Pearce",
    "Steve Streza",
    "Nick Johnson",
    "Nick Gerakines",
    "John Sheehan",
    "David Pollak",
    "Paul Dix",
    "Marcos Tapajós",
    "Stig Brautaset",
    "Chris Coyier",
    "Pete Warden",
    "Jack Moore",
    "Joel Hooks",
    "Christophe Grand",
    "Dann",
    "Steve Jenson",
    "lzyy",
    "Daniel Lopes",
    "Yuki Sonoda (Yugui)",
    "Thomas Aylott",
    "Atsushi Kobayashi",
    "Greg Young",
    "Cameron McEfee",
    "Geoffrey Grosenbach",
    "Dave Rupert",
    "Hooopo",
    "Adam Wiggins",
    "Rich Hickey",
    "Jesse Newland",
    "Claus Wahlers",
    "Garann Means",
    "Dustin Whittle",
    "Indragie Karunaratne",
    "Drew Conway",
    "Joe Stump",
    "Matteo Spinelli",
    "Carlos Brando",
    "Jonathan Rockway",
    "Mike Ash",
    "Hilary Mason",
    "Jimmy Bogard",
    "Tim Holman",
    "Naoya Ito",
    "Francisco Ryan Tolmasky I",
    "Andy Matuschak",
    "Daniel Spiewak",
    "Ola Bini",
    "Ayaka Ikezawa",
    "Devin Ross",
    "Eric Florenzano",
    "Jeffrey Way",
    "Alex Payne",
    "Matt Ranney",
    "Nick Kallen",
    "Cliff Moon",
    "Phil Calçado",
    "Mark Allison",
    "Robin Lu",
    "Jeff Balogh",
    "Richard Jones",
    "Gregg Pollack",
    "Greg Borenstein",
    "Tom Robinson",
    "Ari",
    "Dmitry Baranovskiy",
    "Patrick Wyatt",
    "Josh Susser",
    "yuiseki",
    "Sam de Freyssinet",
    "Tim Burks",
    "Chad Fowler",
    "Fabio Akita",
    "angus croll",
    "Nico Hagenburger",
#    "闲耘™",
    "Grant Paul",
    "Sebastian Riedel",
    "Zach Beane",
    "Yin Wang",
    "Steve Dekorte",
    "Adrian Kosmaczewski",
    "Carlos Villela",
    "Kyle Bragger",
    "bret",
    "Xin Liu",
    "Brandon Kelly",
    "Jay Freeman (saurik)",
    "Julie Ann Horvath",
    "Rob Conery",
    "Dave DeLong",
    "Matthijs Hollemans",
    "Vladimir Kolesnikov",
    "Andrei Zmievski",
    "Erica Sadun",
    "Robert C. Martin",
    "Michael Hoisie",
    "Steve Smith",
    "Ben Orenstein",
    "Michael Koziarski",
    "joe armstrong",
    "Daniel Neighman",
    "Daniel Peebles",
    "Peter Cooper",
    "Guilherme Chapiewski",
    "Brandon Aaron",
    "Christian Van Der Henst S.",
    "Dave Fayram",
    "Nicole Sullivan",
    "Matthew A. Russell",
    "Jamis Buck",
    "Joel Perras",
    "Nathan Borror",
    "Jonnie Hallman",
    "Pat Nakajima",
    "Joshua Hill",
    "Hitoshi Amano",
    "Jim Dovey",
    "Ben Cherry",
    "Micheil Smith",
    "Obie Fernandez",
    "Paulo Silveira",
    "Peter E Higgins",
    "Pratik",
    "Adam Stacoviak",
    "Joe Damato",
    "Matt Gemmell",
    "Paul Neave",
    "Robert Hodgin",
    "Corey Johnson",
    "Rob Hawkes",
    "Ole Zorn",
    "Nodester",
    "Jeremy Keith",
    "Jordan Mechner",
    "Joe Ricioppo",
    "Dan Webb",
    "jawahar",
    "Andrew Gerrand",
    "Loren Brichter",
    "Simon Willison",
    "Leah Culver",
    "Vinicius Teles",
    "mala",
    "Sergey Tikhonov",
    "Matt Todd",
    "kevin",
    "Brendan Eich",
    "Cyril Mottier",
    "Grant Skinner",
    "Blaine Cook",
    "kejun",
    "Masafumi Otsune",
    "Nicolas Haunold",
    "Marcel Molina",
    "Jason Morrissey",
    "Dan Benjamin",
    "Robert Penner",
    "John Freddy Vega",
    "Romain Guy",
    "Adam 'Atomic' Saltsman",
    "Ching-Lan 'digdog' HUANG 黃 青嵐",
    "Bret Taylor",
    "Paul Pajo",
    "Krystal Mok",
    "Yuval Kogman",
    "Peter Boctor",
    "Aaron Newton",
    "The Octocat",
]

NONPEOPLE = [
    "开源中国",
    "California Institute Of Technology",
    "UCSB",
    "MIT",
    "Google",
    "Google Brain",
    "Sony",
#    "Ovid",
    "Account for Github research",
    "cocos2d",
    "20th Century Fox",
    "3Com",
    "3M",
    "7-Up",
    "A Mild",
#    "A.G. Edwards",
    "ABC",
    "ABM Industries",
    "ACT Manufacturing",
    "ADC Telecommunications",
    "ADP",
    "AES",
    "AFLAC",
    "AGCO",
    "AIG",
    "AIMCO",
    "AK Steel Holding",
    "AMC Entertainment",
    "AMR",
    "AOL Time Warner",
    "AT&T",
    "Abbott Laboratories",
    "Abbott Labs",
    "Abbvie",
    "Abercrombie & Fitch",
    "Accenture",
    "Ace Hardware",
    "Acterna",
    "Activision Blizzard",
    "Acuvue",
    "Adams Resources & Energy",
    "Adelphia Communications",
    "Administaff",
    "Adobe",
    "Adobe Systems",
    "Adolph Coors",
    "Advance Auto Parts",
    "AdvancePCS",
    "Advanced Micro Devices",
    "Advantica",
    "Aecom Technology",
    "Aetna",
    "Affiliated Computer Services",
    "Aflac",
    "Agilent Technologies",
    "Agway",
    "Aid Association for Lutherans",
    "Air Products & Chemicals",
    "Airbnb",
    "Airborne",
    "Airgas",
    "Akamai Technologies",
    "Alaska Air Group",
    "Alaska Airlines",
    "Alberto-Culver",
    "Albertson's",
    "Alcoa",
    "Alexander & Baldwin",
    "Alleghany",
    "Allegheny Energy",
    "Allegheny Technologies",
    "Allergan",
    "Allete",
    "Alliant Energy",
    "Allied Waste Industries",
    "Allmerica Financial",
    "Allstate",
    "Alltel",
    "Alphabet",
    "Alphabet Inc.",
    "Alpine Group",
    "Always",
    "AmSouth Bancorp.",
    "Amazon.Com",
    "Amazon.com",
    "Amerada Hess",
    "Amerco",
    "Ameren",
    "America West Holdings",
    "American Airlines",
    "American Axle & Mfg.",
    "American Eagle Outfitters",
    "American Electric Power",
    "American Express",
    "American Family Ins. Group",
    "American Financial Group",
    "American Greetings",
    "American International Group",
    "American Management Systems",
    "American National Insurance",
    "American Power Conversion",
    "American Standard",
    "American Tower",
    "American Water Works",
    "Ameriprise Financial",
    "AmerisourceBergen",
    "Ames Department Stores",
    "Amgen",
    "Amica Mutual Insurance",
    "Amkor Technology",
    "Amphenol",
    "Amtran",
    "Anadarko",
    "Anadarko Petroleum",
    "Analog Devices",
    "Anheuser-Busch",
    "Anixter International",
    "AnnTaylor",
    "Anthem",
    "Aon",
    "Apache",
    "Apple",
    "Apple Computer",
    "Apple Inc",
    "Applera",
    "Applied Industrial Technologies",
    "Applied Materials",
    "Aquila",
    "Aramark",
    "Arch Coal",
    "Archer Daniels Midland",
    "Arkansas Best",
    "Arm & Hammer",
    "Armstrong Holdings",
    "Arrow Electronics",
    "ArvinMeritor",
    "Ashland",
    "Astoria Financial",
    "Atmel",
    "Atmos Energy",
    "Audiovox",
    "Auto-Owners Insurance",
    "AutoNation",
    "AutoZone",
    "Autodesk",
    "Autoliv",
    "Automatic Data Processing",
    "Avaya",
    "Aveeno",
    "Avery Dennison",
    "Avis",
    "Avista",
    "Avnet",
    "Avon",
    "Avon Products",
    "BB&T",
    "BB&T Corp.",
    "BCG",
    "BD",
    "BJ Services",
    "BJ's Wholesale Club",
    "BMC Software",
    "BNSF",
    "BNY Mellon",
    "Baker Hughes",
    "Ball",
    "Bank One Corp.",
    "Bank of America",
    "Bank of America Corp.",
    "Bank of New York Co.",
    "Banknorth Group",
    "Banta",
    "Barnes & Noble",
    "Bath and Body Works",
    "Bausch & Lomb",
    "Baxter International",
    "Bear Stearns",
    "Beazer Homes USA",
    "Beckman Coulter",
    "Becton Dickinson",
    "Bed Bath & Beyond",
    "Belk",
    "Bell Microproducts",
    "BellSouth",
    "Belo",
    "Bemis",
    "Benchmark Electronics",
    "BeneFit",
    "Berkshire Hathaway",
    "Best Buy",
    "Bethlehem Steel",
    "Beverly Enterprises",
    "Big Lots",
    "Biogen",
    "Black & Decker",
    "Black Hills",
    "Black+Decker",
    "Blackrock",
    "Blackstone",
    "Bloomberg",
    "Blyth",
    "Boeing",
    "Boise Cascade",
    "Bond Street",
    "Booking Holdings",
    "Booz Allen Hamilton",
    "Borders Group",
    "BorgWarner",
    "Boston Scientific",
    "Bowater",
    "Briggs & Stratton",
    "Brightpoint",
    "Brinker International",
    "Bristol-Myer Squibb",
    "Bristol-Myers Squibb",
    "Broadcom",
    "Broadcom Inc.",
    "Broadridge",
    "Broadwing",
    "Brown Shoe",
    "Brown-Forman",
    "Brunswick",
    "Bud Light",
    "Budget Group",
    "Budweiser",
    "Buffalo Wild Wings",
    "Buick",
    "Burger King",
    "Burlington Coat Factory",
    "Burlington Industries",
    "Burlington Northern Santa Fe",
    "Burlington Resources",
    "Busch",
    "C.H. Robinson Worldwide",
    "C.R. Bard",
    "CA Technologies",
    "CBRE",
    "CBRL Group",
    "CBS",
    "CDI",
    "CDW",
    "CDW Computer Centers",
    "CH2M Hill",
    "CITRIX",
    "CKE Restaurants",
    "CME",
    "CMGI",
    "CMS Energy",
    "CNF",
    "CSC",
    "CSK Auto",
    "CSX",
    "CUNA Mutual Group",
    "CVS",
    "CVS Caremark",
    "Cablevision Systems",
    "Cabot",
    "Cadence Design Systems",
    "Cadillac",
    "Calpine",
    "Calvin Klein",
    "Camel",
    "Campbell Soup",
    "Campbell's",
    "Capital One",
    "Capital One Financial",
    "Cardinal Health",
    "Carefree",
    "Caremark Rx",
    "Carlisle",
    "Carmax",
    "Carnival Cruise Lines",
    "Carpenter Technology",
    "Carrier",
    "Casey's General",
    "Casey's General Stores",
    "Caterpillar",
    "Celgene",
    "CellStar",
    "Cendant",
    "Cenex Harvest States",
    "Centene Corporation",
    "Centex",
    "CenturyTel",
    "Centurylink",
    "Ceridian",
    "Cerner Corp",
    "Champion Enterprises",
    "Charles Schwab",
    "Charming Shoppes",
    "Charter Communications",
    "Charter One Financial",
    "Chase",
    "Cheetos",
    "Chesapeake Energy",
    "Chesterfield",
    "Chevrolet",
    "Chevron",
    "ChevronTexaco",
    "Chipotle",
    "Chiquita Brands International",
    "Chubb",
    "Ciena",
    "Cigna",
    "Cincinnati Financial",
    "Cinemark",
    "Cinergy",
    "Cintas",
    "Circuit City Stores",
    "Cisco",
    "Cisco Systems",
    "Citi",
    "Citigroup",
    "Citizens",
    "Citizens Communications",
    "Clear Channel Communications",
    "Clinique",
    "Clorox",
    "Coach",
    "Coca-Cola",
    "Coca-Cola Enterprises",
    "Cognizant",
    "Colgate",
    "Colgate-Palmolive",
    "Collins & Aikman",
    "Comcast",
    "Comdisco",
    "Comedy Central",
    "Comerica",
    "Comfort Systems USA",
    "CommScope",
    "Commercial Metals",
    "Community Health Systems",
    "Compaq Computer",
    "Compass Bancshares",
    "Computer Associates Intl.",
    "Computer Sciences",
    "Compuware",
    "Comverse Technology",
    "ConAgra",
    "Concord EFS",
    "Conectiv",
    "Conoco",
    "Conocophillips",
    "Conseco",
    "Consolidated Edison",
    "Consolidated Freightways",
    "Constellation Brands",
    "Constellation Energy",
    "Continental Airlines",
    "Convergys",
    "Converse",
    "Cooper Cameron",
    "Cooper Industries",
    "Cooper Tire & Rubber",
    "Coors Light",
    "Copenhagen",
    "Core-Mark International",
    "Corn Products International",
    "Corning",
    "Costco",
    "Costco Wholesale",
    "Countrywide Credit Industries",
    "Courtyard",
    "Coventry Health Care",
    "Cox Communications",
    "Crane",
    "Crompton",
    "Crown Cork & Seal",
    "Cummins",
    "Cytec Industries",
    "D&K Healthcare Resources",
    "D.R. Horton",
    "DPL",
    "DQE",
    "DST Systems",
    "DTE Energy",
    "DXC Technology",
    "DaVita",
    "Daisytek International",
    "Dana",
    "Danaher",
    "Darden Restaurants",
    "Davita",
    "Dean Foods",
    "Deere",
    "Del Monte Foods",
    "Dell",
    "Dell Computer",
    "Dell Technologies",
    "Deloitte",
    "Delphi",
    "Delta",
    "Delta Air Lines",
    "Deluxe",
    "Devon",
    "Devon Energy",
    "Di Giorgio",
    "Dial",
    "Dick's Sporting",
    "Diebold",
    "Dillard's",
    "Dimon",
    "Direct Energy",
    "Discover",
    "Discovery",
    "Dish Network",
    "Dodge",
    "Dole Food",
    "Dollar General",
    "Dollar Tree",
    "Dollar Tree Stores",
    "Dominion",
    "Dominion Resources",
    "Domino's",
    "Domino's Pizza",
    "Doritos",
    "Double Tree",
    "Dover",
    "Dow",
    "Dow Chemical",
    "Dow Jones",
    "Dr Pepper",
    "Dreyer's Grand Ice Cream",
    "Du Pont",
    "Duke Energy",
    "Dun & Bradstreet",
    "Dunkin' Donuts",
    "Dura Automotive Systems",
    "DynCorp",
    "Dynegy",
    "E*Trade Group",
    "E.I. du Pont de Nemours",
    "E.W. Scripps",
    "EGL",
    "EMC",
    "EOG Resources",
    "ESPN",
    "EarthLink",
    "Earthgrains",
    "Eastman Chemical",
    "Eastman Kodak",
    "Eaton",
    "Echostar Communications",
    "Ecolab",
    "Ecolab Inc",
    "Edison International",
    "El Paso",
    "Electronic Arts",
    "Electronic Data Systems",
    "Eli Lilly",
    "Eli Lilly & Co",
    "Emcor Group",
    "Emerson Electric",
    "Encompass Services",
    "Energizer Holdings",
    "Energy East",
    "Enfamil",
    "Engelhard",
    "Enron",
    "Entergy",
    "Enterprise",
    "Enterprise Products",
    "Eog Resources",
    "Equifax",
    "Equinix",
    "Equitable Resources",
    "Equity Office Properties",
    "Equity Residential Properties",
    "Erie Insurance Group",
    "Esso",
    "Estee Lauder",
    "Estée Lauder",
    "Euro Sport",
    "Exelon",
    "Exide Technologies",
    "Expedia",
    "Expedia.com",
    "Expeditors Intl of Washington",
    "Experian",
    "Express Scripts",
    "Exxon",
    "Exxon Mobil",
    "ExxonMobil",
    "FIS",
    "FMC",
    "FPL Group",
    "Facebook",
    "Fairchild Semiconductor Intl.",
    "Family Dollar",
    "Family Dollar Stores",
    "Fannie Mae",
    "Fanta",
    "Farmland Industries",
    "FedEx",
    "Federal-Mogul",
    "Federated Department Stores",
    "Federated Mutual Insurance",
    "FelCor Lodging",
    "Ferro",
    "Fidelity National Financial",
    "Fifth Third Bancorp",
    "Fifth Third Bank",
    "First American Corp.",
    "First Data",
    "First National of Nebraska",
    "First Tennessee National Corp.",
    "FirstEnergy",
    "Fiserv",
    "Fiserv.",
    "Fisher Scientific International",
    "FleetBoston Financial",
    "Fleetwood Enterprises",
    "Fleming",
    "Flex",
    "Florida Power & Light",
    "Flowers Foods",
    "Flowserve",
    "Fluor",
    "Foamex International",
    "Folgers",
    "Food 4 Less",
    "Foot Locker",
    "Footstar",
    "Ford",
    "Ford Motor",
    "Forest Laboratories",
    "Fortune Brands",
    "Fortune International",
    "Fossil",
    "Foster Wheeler",
    "Fox",
    "Franklin Resources",
    "Franklin Templeton Investments",
    "Freddie Mac",
    "Freeport-McMoRan Copper & Gold",
    "Frontier Communication",
    "Frontier Oil",
    "Furniture Brands International",
    "GATX",
    "GMC",
    "Gannett",
    "Gap",
    "Garden Inn",
    "Gateway",
    "Gatorade",
    "Geico",
    "Gemstar-TV Guide International",
    "GenCorp",
    "General Cable Corporation",
    "General Dynamics",
    "General Electric",
    "General Mills",
    "General Motors",
    "Genesis Health Ventures",
    "Gentek",
    "Gentiva Health Services",
    "Genuine Parts",
    "Genuity",
    "Genzyme",
    "Georgia Gulf",
    "Georgia-Pacific",
    "Gilead Sciences",
    "Gillette",
    "Gold Kist",
    "Golden State Bancorp",
    "Golden West Financial",
    "Goldman Sachs",
    "Goldman Sachs Group",
    "Goodrich",
    "Goody's Family Clothing",
    "Goodyear",
    "Goodyear Tire & Rubber",
    "Google",
    "Google Brain",
    "Grainger",
    "Granite",
    "Graybar Electric",
    "Great Lakes Chemical",
    "Great Plains Energy",
    "Greenpoint Financial",
    "Greif Bros.",
    "Grey Global",
    "Grey Goose",
    "Grizzly",
    "Group 1 Automotive",
    "Guardian Life Ins. Co. of America",
    "Guidant",
    "H&R Block",
#    "H.B. Fuller",
#    "H.J. Heinz",
    "HBO",
    "HCA",
    "HCL",
    "HP",
    "HP Inc.",
    "HYATT",
    "Halliburton",
    "Hampton Inn",
    "Handleman",
    "Hanes",
    "Harley-Davidson",
    "Harleysville Mutual Insurance",
    "Harman",
    "Harman International",
    "Harman Intl. Industries",
    "Harrah's Entertainment",
    "Harris",
    "Harris Corp",
    "Harsco",
    "Hartford Financial Services",
    "Hasbro",
    "Hawaiian Electric Industries",
    "Head & Shoulders",
    "Health Management Associates",
    "Health Net",
    "HealthNet",
    "Healthsouth",
    "Heinz",
    "Henry Schein",
    "Hercules",
    "Heritage",
    "Herman Miller",
    "Hershey Foods",
    "Hershey's",
    "Hertz",
    "Hewlett Packard Enterprise",
    "Hewlett-Packard",
    "Hibernia Corp.",
    "Hillenbrand Industries",
    "Hilton",
    "Hilton Hotels",
    "Holiday Inn",
    "Hollywood Entertainment",
    "Home Depot",
    "Hon Industries",
    "Honeywell",
    "Honeywell International",
    "Hormel",
    "Hormel Foods",
    "Host Marriott",
    "Household International",
    "Hovnanian Enterprises",
    "Hub Group",
    "Hubbell",
    "Huggies",
    "Hughes Supply",
    "Humana",
    "Huntington Bancshares",
    "IBM",
    "IDT",
    "IMC Global",
    "IMS Health",
    "IT Group",
    "ITT Industries",
    "ITW",
    "Idacorp",
    "Ikon Office Solutions",
    "Illinois Tool Works",
    "Imperial Sugar",
    "Infosys",
    "Ingersoll Rand",
    "Ingles Markets",
    "Ingram Micro",
    "Insight Enterprises",
    "Integrated Electrical Services",
    "Intel",
    "International Game Technology",
    "International Multifoods",
    "International Paper",
    "Interpublic Group",
    "Interstate Bakeries",
    "Intl. Business Machines",
    "Intl. Flavors & Fragrances",
    "Intuit",
    "Invesco",
    "Iron Mountain",
    "Ivax",
    "J.B. Hunt Transport Services",
    "J.C. Penney",
    "J.P. Morgan",
    "J.P. Morgan Chase & Co.",
    "JDS Uniphase",
    "Jabil",
    "Jabil Circuit",
    "Jack Daniel's",
    "Jack in the Box",
    "Jackson National Life Insurance Company",
    "Jacobs Engineering Group",
    "Jeep",
    "Jefferson-Pilot",
    "Jetblue Airways",
    "Jo Ann Stores",
    "John Deere",
    "John Hancock Financial Services",
    "Johnson & Johnson",
    "Johnson Controls",
    "Johnson's",
    "Jones Apparel Group",
    "Juniper Networks",
    "KB Home",
    "KFC",
    "KPMG Consulting",
    "Kellogg",
    "Kellogg's",
    "Kellwood",
    "Kelly Services",
    "Kemet",
    "Kennametal",
    "Kent",
    "Kerr-McGee",
    "KeyBank",
    "KeyCorp",
    "KeySpan",
    "Kimball International",
    "Kimberly-Clark",
    "Kinder Morgan",
    "Kindred Healthcare",
    "KitchenAid",
    "Kla-Tencor",
    "Kleenex",
    "Kmart",
    "Knight-Ridder",
    "Knights of Columbus",
    "Kohl's",
    "Kraft",
    "Kroger",
    "L&M",
    "L-3 Communications",
    "LSI Logic",
    "LTV",
    "La-Z-Boy",
    "LabCorp",
    "Laboratory Corp. of America",
    "Lam Research",
    "LandAmerica Financial Group",
    "Lands' End",
    "Landstar System",
    "Lay's",
    "Le Petit Marseillais",
    "Lear",
    "Legg Mason",
    "Leggett & Platt",
    "Lehman Brothers Holdings",
    "Leidos Holdings",
    "Lennar",
    "Lennox International",
    "Level (3)",
    "Level 3 Communications",
    "Levi Strauss",
    "Lexisnexis",
    "Lexmark International",
    "Liberty Mutual Insurance Group",
    "Limited",
    "Lincoln",
    "Lincoln National",
    "Linens'n Things",
    "Linkedin",
    "Listerine",
    "Lithia Motors",
    "Liz Claiborne",
    "Lockheed Martin",
    "Loews",
    "Longs Drug Stores",
    "Louisiana-Pacific",
    "Lowe's",
    "Lubrizol",
    "Lucent Technologies",
    "Lucky Strike",
    "Lutheran Brotherhood",
    "Lyondell Chemical",
    "Lyondellbasell",
    "M & T Bank Corp.",
    "MBNA",
    "MDC Holdings",
    "MDU Resources Group",
    "MGIC Investment",
    "MGM Mirage",
    "MONY Group",
    "MPS Group",
    "MTV",
    "Mac",
    "Macy's",
    "Magellan Health Services",
    "Mail-Well",
    "Mandalay Resort Group",
    "Manor Care",
    "Manpower",
    "Marathon Oil",
    "Marathon Petroleum",
    "Mariner Post-Acute Network",
    "Markel",
    "Marlboro",
    "Marriot",
    "Marriott International",
    "Mars",
    "Marsh",
    "Marsh & McLennan",
    "Marsh Supermarkets",
    "Marshall & Ilsley Corp.",
    "Marshalls",
    "Martin Marietta Materials",
    "MasTec",
    "Masco",
    "Mass. Mutual Life Insurance",
    "Massey Energy",
    "Mastercard",
    "Mattel",
    "Maxim Integrated Products",
    "Maxtor",
    "Maxxam",
    "May Department Stores",
    "Maybelline",
    "Maytag",
    "McCormick",
    "McDonald's",
    "McGraw-Hill",
    "McKesson",
    "McKinsey",
    "McLane",
    "McLeodUSA",
    "Mckesson",
    "MeadWestvaco",
    "Medtronic",
    "Mellon Financial Corp.",
    "Men's Wearhouse",
    "Merck",
    "Mercury General",
    "Merrill Lynch",
    "MetLife",
    "Metaldyne",
    "Metals USA",
    "Metlife",
    "Metris",
    "Metro-Goldwyn-Mayer",
    "Michael Kors",
    "Michaels",
    "Michaels Stores",
    "Micron Technology",
    "Microsoft",
    "Mid Atlantic Medical Services",
    "Milacron",
    "Millennium Chemicals",
    "Miller Lite",
    "Minnesota Life Ins.",
    "Minnesota Mining & Mfg.",
    "Minute Maid",
    "Mirant",
    "Mobil",
    "Mohawk Industries",
    "Molex",
    "Monsanto",
    "Monster",
    "Moody's",
    "Morgan Stanley",
    "Morgan Stanley Dean Witter",
    "Motorola",
    "Motorola Solutions",
    "Mountain Dew",
    "Murphy Oil",
    "Mutual of Omaha Insurance",
    "NAPA",
    "NBC",
    "NCR",
    "NCR Inc.",
    "NETFLIX",
    "NOV",
    "NSTAR",
    "NTL",
    "NVR",
    "Nabors Industries",
    "Nacco Industries",
    "Nash Finch",
    "National City Corp.",
    "National Commerce Financial",
    "National Fuel Gas",
    "National Oilwell",
    "National Rural Utilities Cooperative",
    "National Semiconductor",
    "National Service Industries",
    "Nationwide",
    "Nature Valley",
    "Navistar International",
    "Neiman Marcus",
    "NetApp",
    "Netapp",
    "Neutrogena",
    "New Jersey Resources",
    "New York Life Insurance",
    "New York Times",
    "Newell Rubbermaid",
    "Newmont Mining",
    "Newport",
    "Nextel Communications",
    "NiSource",
    "Nicor",
    "Nielsen",
    "Nike",
    "Noble Affiliates",
    "Nordstrom",
    "Norfolk",
    "Norfolk Southern",
    "Nortek",
    "North Fork Bancorp.",
    "NorthWestern",
    "Northeast Utilities",
    "Northern Trust Corp.",
    "Northrop Grumman",
    "Northwest Airlines",
    "Northwestern Mutual",
    "Norwegian Cruise",
    "Novellus Systems",
    "Nucor",
    "Nvidia",
    "O'Reilly Auto Parts",
    "OGE Energy",
    "OM Group",
    "ONEOK",
    "Occidental Petroleum",
    "Ocean Energy",
    "Office Depot",
    "OfficeMax",
    "Oglethorpe Power",
    "Ohio Casualty",
    "Olay",
    "Old Navy",
    "Old Republic International",
    "Old Spice",
    "Olin",
    "Omnicare",
    "Omnicom Group",
    "On Semiconductor",
    "OneAmerica Financial",
    "Oneok",
    "Optimum",
    "Optum",
    "Oracle",
    "Oral-B",
    "Oscar Mayer",
    "Oshkosh Truck",
    "Otis",
    "Outback Steakhouse",
    "Owens & Minor",
    "Owens Corning",
    "Owens-Illinois",
    "Oxford Health Plans",
    "Oxy",
    "PAYPAL",
    "PC Connection",
    "PG&E",
    "PG&E Corp.",
    "PNC",
    "PNC Financial Services Group",
    "PNM Resources",
    "PPG Industries",
    "PPL",
    "PSEG",
    "PSS World Medical",
    "PWC",
    "Paccar",
    "PacifiCare Health Systems",
    "Pacific Century Financial",
    "Pacific LifeCorp",
    "Packaging Corp. of America",
    "Pactiv",
    "Pall",
    "Pall Mall",
    "Palmolive",
    "Pampers",
    "Panera Bread",
    "Pantene",
    "Pantry",
    "Park Place Entertainment",
    "Parker Hannifin",
    "Parker-Hannifin",
    "Parliament",
    "Pathmark Stores",
    "Patterson Companies",
    "PayPal Holdings, Inc.",
    "Paychex",
    "Payless Shoesource",
    "Penn Traffic",
    "Pennzoil-Quaker State",
    "Pentair",
    "PeopleSoft",
    "Peoples Energy",
    "Pep Boys-Manny, Moe & Jack",
    "Pepsi",
    "Pepsi Bottling",
    "PepsiAmericas",
    "PepsiCo",
    "Performance Food Group",
    "Perini",
    "PerkinElmer",
    "Perot Systems",
    "Petco Animal Supplies",
    "Peter Kiewit Sons'",
    "PetsMart",
    "Pfizer",
    "Phar-Mor",
    "Pharmacia",
    "Phelps Dodge",
    "Philadelphia Insurance Companies",
    "Philip Morris",
    "Phillips 66",
    "Phillips Petroleum",
    "Phillips-Van Heusen",
    "Phoenix Companies",
    "Pier 1 Imports",
    "Pilgrim's Pride",
    "Pinnacle West Capital",
    "Pioneer Natural",
    "Pioneer-Standard Electronics",
    "Pitney Bowes",
    "Pittston",
    "Pizza Hut",
    "Plains All American Pipeline",
    "Polaris Inds",
    "Polaris Industries",
    "Polo Ralph Lauren",
    "PolyOne",
    "Popular",
    "Potlatch",
    "Potomac Electric Power",
    "Pratt & Whitney",
    "Praxair",
    "Precision Castparts",
    "Premcor",
    "Pride International",
    "Primedia",
    "Principal Financial",
    "Pro-Fac Cooperative",
    "Procter & Gamble",
    "Progress Energy",
    "Progressive",
    "Protective Life",
    "Provident Financial Group",
    "Providian Financial",
    "Prudential (US)",
    "Prudential Financial",
    "Public Service Enterprise Group",
    "Publix",
    "Publix Super Markets",
    "Puget Energy",
    "Pulte Homes",
    "Purina",
    "QVC",
    "Quaker",
    "Qualcomm",
    "Quanta Services",
    "Quantum",
    "Quest Diagnostics",
    "Questar",
    "QuickBooks",
    "Quintiles Transnational",
    "Qurate Retail",
    "Qwest Communications",
    "R.J. Reynolds Tobacco",
    "R.R. Donnelley & Sons",
    "RGS Energy Group",
    "RPM",
    "RR Donnelley",
    "RadioShack",
    "Ramada",
    "Raymond James",
    "Raymond James Financial",
    "Raytheon",
    "Reader's Digest Association",
    "Red Hat",
    "Reebok International",
    "Reese's",
    "Regions Financial",
    "Regions Financial Corporation",
    "Regis",
    "Reliance Steel & Aluminum",
    "Reliant Energy",
    "Rent A Center",
    "Republic Services",
    "Revlon",
    "Rite Aid",
    "Riverwood Holding",
    "Roadway",
    "Robert Half International",
    "Rock-Tenn",
    "Rockwell Automation",
    "Rockwell Collins",
    "Rohm & Haas",
    "Ross Dress For Less",
    "Ross Stores",
    "Roundy's",
    "Royal Caribbean International",
    "Ruddick",
    "Ryder System",
    "Ryerson Tull",
    "Ryland Group",
    "SBC Communications",
    "SCI Systems",
    "SPX",
    "Sabre",
    "Sabre Holdings",
    "Safeco",
    "Safeguard Scientifics",
    "Safeway",
    "Saks",
    "Sally Beauty",
    "Sam's Club",
    "Sanderson Farms",
    "Sandisk",
    "Sanmina Corporation",
    "Sanmina-SCI",
    "Sara Lee",
    "Scana",
    "Schering-Plough",
    "Schlumberger",
    "Scholastic",
    "Science Applications International",
    "Science Applications Intl.",
    "Scientific-Atlanta",
    "Scotts Company",
    "Seaboard",
    "Seagate",
    "Seagate Technology",
    "Sealed Air",
    "Sealy",
    "Sears Roebuck",
    "Sempra Energy",
    "Sentry Insurance Group",
    "Sequa",
    "Service Corp. International",
    "ServiceMaster",
    "Servicenow",
    "Shaw Group",
    "Sheraton",
    "Sherwin-Williams",
    "Shopko Stores",
    "Siebel Systems",
    "Sierra Health Services",
    "Sierra Pacific Resources",
    "Silgan Holdings",
    "Silicon Graphics",
    "Simon Property Group",
    "Sirius",
    "Sirona",
    "Skechers",
    "Skoal",
    "Smith International",
    "Smithfield Foods",
    "Smurfit-Stone Container",
    "Snap-On",
    "Snapchat",
    "Software Spectrum",
    "Solectron",
    "Solutia",
    "Sonic Automotive",
    "Sonoco Products",
    "SouthTrust Corp.",
    "Southern",
    "Southern Company",
    "Southern States Coop.",
    "Southern Union",
    "Southwest Airlines",
    "Southwest Gas",
    "Sovereign Bancorp",
    "Spartan Stores",
    "Spectrum",
    "Spherion",
    "Sports Authority",
    "Sprint",
    "Sprite",
    "St Jude Medical",
    "St. Jude Medical",
    "St. Paul Cos.",
    "Staff Leasing",
    "StanCorp Financial",
    "Standard Pacific",
    "Standard Register",
    "Stanley Works",
    "Staples",
    "Starbucks",
    "Starwood Hotels & Resorts",
    "State Farm Insurance Cos.",
    "State Street",
    "State Street Corp.",
    "Stater Bros. Holdings",
    "Steelcase",
    "Stein Mart",
    "Stewart & Stevenson Services",
    "Stewart Information Services",
    "Stilwell Financial",
    "Storage Technology",
    "Stryker",
    "Subway",
    "Sun Healthcare Group",
    "Sun Microsystems",
    "SunGard Data Systems",
    "SunTrust Banks",
    "Sundown",
    "Sunoco",
    "Supervalu",
    "Swift Transportation",
    "Symantec",
    "Symbol Technologies",
    "Synovus Financial Corp.",
    "Sysco",
    "Systemax",
    "T.J. Maxx",
    "TBS",
    "TCF Financial Corp.",
    "TCS",
    "TECO Energy",
    "TIAA-CREF",
    "TJX",
    "TMP Worldwide",
    "TNT",
    "TRW",
    "TXU",
    "Taco Bell",
    "Target",
    "Tech Data",
    "Tecumseh Products",
    "Tektronix",
    "Teleflex",
    "Telephone & Data Systems",
    "Tellabs",
    "Temple-Inland",
    "Tenet Health",
    "Tenet Healthcare",
    "Tenneco Automotive",
    "Teradyne",
    "Terex",
    "Tesla Motors",
    "Tesoro",
    "Tesoro Petroleum",
    "Texas Industries",
    "Texas Instruments",
    "Textron",
    "The Hartford",
    "The North Face",
    "Thermo Electron",
    "Thermo Fisher Scientific",
    "Thomas & Betts",
    "Thomson Reuters",
    "Tide/Ariel",
    "Tiffany & Co",
    "Tiffany & Co.",
    "Timberland",
    "Timken",
    "Toll Brothers",
    "Tommy Hilfiger",
    "Torchmark",
    "Toro",
    "Tower Automotive",
    "Toys R Us",
    "Toys `R` Us",
    "Tracfone",
    "Tractor Supply",
    "Trans World Entertainment",
    "TransMontaigne",
    "Transocean Sedco Forex",
    "TravelCenters of America",
    "Travelers",
    "Triad Hospitals",
    "Tribune",
    "Tricon Global Restaurants",
    "Trident",
    "Trigon Healthcare",
    "Trinity Industries",
    "Tripadvisor",
    "Tropicana",
    "TruServ",
    "Trump Hotels & Casino Resorts",
    "TurboTax",
    "Twitter",
    "Tyson",
    "Tyson Foods",
    "U.S. Bancorp",
    "U.S. Industries",
    "UAL",
    "UGI",
    "UNIVERSAL",
    "UPS",
    "URS",
    "US Airways Group",
    "US Oncology",
    "USA Education",
    "USA Networks",
    "USFreightways",
    "USG",
    "UST",
    "Uber",
    "Uber Technologies",
    "Ulta Salon Cosmetics",
    "Under Armour",
    "Unified Western Grocers",
    "Union Central Life",
    "Union Pacific",
    "Union Planters Corp.",
    "Unisource Energy",
    "Unisys",
    "United Airlines",
    "United Auto Group",
    "United Defense Industries",
    "United Parcel Service",
    "United Rentals",
    "United Services Automobile Assn.",
    "United Stationers",
    "United Technologies",
    "UnitedHealth Group",
    "Unitrin",
    "Universal",
    "Universal Forest Products",
    "Universal Health Services",
    "Unocal",
    "Unova",
    "UnumProvident",
    "VF",
    "VMware",
    "Valeant",
    "Valero",
    "Valero Energy",
    "Valspar",
    "Value City",
    "Varco International",
    "Vectren",
    "Vegas Sands",
    "Ventas",
    "Veritas Software",
    "Verizon",
    "Verizon Communications",
    "Viacom",
    "Viad",
    "Viasystems Group",
    "Viceroy",
    "Victoria's Secret",
    "Visa",
    "Vishay Intertechnology",
    "Visteon",
    "Volt Information Sciences",
    "Vulcan Materials",
    "W.R. Berkley",
    "W.R. Grace",
    "W.W. Grainger",
    "WGL Holdings",
    "WPS Resources",
    "Wachovia Corp.",
    "Wackenhut",
    "Wal-Mart Stores",
    "Walgreen",
    "Walgreens",
    "Wallace Computer Services",
    "Walmart",
    "Walt Disney",
    "Walter Industries",
    "Warner Bros.",
    "Washington Mutual",
    "Washington Post",
    "Waste Management",
    "Watsco",
    "Wayfair",
    "Weatherford International",
    "Weis Markets",
    "WellPoint Health Networks",
    "Wells Fargo",
    "Welltower Inc.",
    "Wendy's",
    "Wendy's International",
    "Werner Enterprises",
    "Wesco International",
    "WestPoint Stevens",
    "Western Digital",
    "Western Gas Resources",
    "Western Resources",
    "Western Union",
    "Westin",
    "Weyerhaeuser",
    "Whirlpool",
    "Whole Foods",
    "Whole Foods Market",
    "Willamette Industries",
    "Williams",
    "Williams-Sonoma",
    "Windstream",
    "Winn-Dixie Stores",
    "Wisconsin Energy",
    "Wm. Wrigley Jr.",
    "World Fuel Services",
    "WorldCom",
    "Worthington Industries",
    "Wrigley's",
    "Wyeth",
    "Wyndham",
    "Wyndham International",
    "Wynn Resorts",
    "XO Communications",
    "Xbox",
    "Xcel Energy",
    "Xerox",
    "Xerox Holdings",
    "Xfinity",
    "Xilinx",
    "Yahoo!",
    "Yellow",
    "Yoplait",
    "York International",
    "YouTube",
    "Zale",
    "Zimmer",
    "Zions Bancorp.",
    "booking.com",
    "eBay",
    "eMerge Interactive",
    "f5",
    "flextronics",
    "nrg",
    "salesforce",
    "salesforce.com",
]

def test_is_person():
    for name in PEOPLE:
        assert is_person(name)
    for name in NONPEOPLE:
        assert not is_person(name)



'''
exceptions.py: exceptions for IGA

This file is part of https://github.com/caltechlibrary/iga/.

Copyright (c) 2022-2023 by the California Institute of Technology.  This code
is open-source software released under a BSD-type license.  Please see the
file "LICENSE" for more information.
'''


# Base class.
# .............................................................................
# The base class makes it possible to use a single test to distinguish between
# exceptions generated by IGA and exceptions generated by something else.

class IGAException(Exception):
    '''Base class for IGA exceptions.'''


# Exception classes.
# .............................................................................

class GitHubError(IGAException):
    '''GitHub returned an error.'''


class InvenioRDMError(IGAException):
    '''InvenioRDM returned an error.'''


class MissingData(IGAException):
    '''Could not obtain all the required data to create an InvenioRDM record.'''


class RecordNotFound(IGAException):
    '''Could not find the requested record in the InvenioRDM server.'''


class InternalError(IGAException):
    '''An internal error occurred in IGA.'''



# =============================================================================
# @file    test_id_utils.py
# @brief   Py.test cases for parts of id_utils.py
# @created 2023-03-02
# @license Please see the file named LICENSE in the project directory
# @website https://github.com/caltechlibrary/iga
# =============================================================================

from iga.id_utils import (
    contains_pmcid,
    detected_id,
    normalize_pmcid,
    recognized_scheme,
    is_invenio_rdm,
)

sample_ids = [
    ('arXiv:2012.13117v1'                                 , 'arxiv'),
    ('10.48550/arXiv.2012.13117'                          , 'doi'),
    ('PMC4908318'                                         , 'pmcid'),
    ('26861819'                                           , 'pmid'),
    ('18183754'                                           , 'pmid'),
    ('10.1093/bioinformatics/btw056'                      , 'doi'),
    ('10.1007/978-1-4939-3283-2_19'                       , 'doi'),
    ('10.1103/PhysRevD.90.124021'                         , 'doi'),
    ('26677194'                                           , 'pmid'),
    ('978-0982477373'                                     , 'isbn'),
    ('9780898714128'                                      , 'isbn'),
    ('9781979381208'                                      , 'isbn'),
    ('978-65-87773-12-4'                                  , 'isbn'),
    ('04dkp9463'                                          , 'ror'),
    ('swh:1:cnt:94a9ed024d3859793618152ea559a168bbcbb5e2' , 'swh'),
    ('bknz4-bch35'                                        , 'rdm'),
    ('ry4vm-wny44'                                        , 'rdm'),
    ('https://a.b/records/ry4vm-wny44'                    , 'rdm'),
    ('https://a.b/records/jx193-a0e45?preview=1'          , 'rdm'),
    ('https://a.b/uploads/ry4vm-wny44'                    , 'rdm'),
    ('https://a.b/something/ry4vm-wny44'                  , 'url'),
]


def test_contains_pmcid():
    for _id, scheme in sample_ids:
        if scheme == 'pmcid':
            assert contains_pmcid(_id)


def test_normalize_pmcid():
    assert normalize_pmcid('pmc4908318') == 'PMC4908318'


def test_recognized_scheme():
    for _id, scheme in sample_ids:
        assert recognized_scheme(_id) == scheme


sample_unnormalized_ids = [
    ('http://orcid.org/0000-0001-9105-5960'      , '0000-0001-9105-5960'),
    ('https://doi.org/10.5281/zenodo.1095472'    , '10.5281/zenodo.1095472'),
    ('https://ror.org/027m9bs27'                 , '027m9bs27'),
    ('10.1088/0264-9381/26/22/225003'            , '10.1088/0264-9381/26/22/225003'),
    ('PMCID; PMC4908318'                         , ''),
    ('22110403'                                  , '22110403'),
    ('pmc4908318'                                , 'PMC4908318'),
    ('PMC3217025'                                , 'PMC3217025'),
    ('https://ror.org/04dkp9463'                 , '04dkp9463'),
    ('https://a.b/records/ry4vm-wny44'           , 'ry4vm-wny44'),
    ('https://a.b/records/ry4vm-wny44?preview=1' , 'ry4vm-wny44'),
    ('https://a.b/records/ry4vm-wny44/draft'     , 'ry4vm-wny44'),
    ('https://a.b/uploads/ry4vm-wny44'           , 'ry4vm-wny44'),
    # If it's not detected as an RDM record id   , it's considered to be just a URL.
    ('https://a.b/something/ry4vm-wny44'         , 'https://a.b/something/ry4vm-wny44'),
]


def test_detected_id():
    for text, id_ in sample_unnormalized_ids:
        assert detected_id(text) == id_


def test_is_invenio_rdm():
    assert is_invenio_rdm('https://a.b/records/ry4vm-wny44')
    assert not is_invenio_rdm('https://a.b/something/ry4vm-wny44')



# =============================================================================
# @file    test_licenses.py
# @brief   Py.test cases for parts of licenses.py
# @created 2023-03-02
# @license Please see the file named LICENSE in the project directory
# @website https://github.com/caltechlibrary/iga
# =============================================================================

from iga.licenses import LICENSES, LICENSE_URLS


def test_licenses():
    assert len(LICENSES) > 100
    assert 'MIT' in LICENSES
    assert LICENSES['MIT'].title == 'MIT License'
    assert LICENSES['MIT'].url == 'https://spdx.org/licenses/MIT'


def test_license_urls():
    assert 'https://spdx.org/licenses/MIT' in LICENSE_URLS
    assert 'https://opensource.org/licenses/MIT' in LICENSE_URLS
    assert LICENSE_URLS['https://spdx.org/licenses/MIT'] == 'MIT'
    assert 'https://spdx.org/licenses/BSD-3-Clause' in LICENSE_URLS



'''
__main__.py: main entry point for IGA.

This file is part of https://github.com/caltechlibrary/iga/.

Copyright (c) 2022-2023 by the California Institute of Technology.  This code
is open-source software released under a BSD-type license.  Please see the
file "LICENSE" for more information.
'''

import sys
if sys.version_info <= (3, 9):
    print('IGA requires Python version 3.9 or higher,')
    print('but the current version of Python is ' + str(sys.version_info.major)
          + '.' + str(sys.version_info.minor) + '.')
    sys.exit(1)

from iga.cli import cli


# The following entry point definition is for the console_scripts keyword
# option to setuptools.  The entry point for console_scripts has to be a
# function that takes zero arguments.
def console_scripts_main():
    cli()


# The following allows users to invoke this using "python3 -m iga".
if __name__ == '__main__':
    cli()



import json5
import os
from   os import path
from   sidetrack import log
from   unittest import mock

from iga.ror import name_from_ror


# Mocks
# .............................................................................

here    = path.dirname(path.abspath(__file__))
ror_dir = path.join(here, 'data/ror-examples/')


def mocked_ror_data(rorid):
    log(f'returing mocked ROR data for {rorid}')
    ror_filename = rorid + '.json'
    with open(path.join(ror_dir, ror_filename), 'r') as f:
        return json5.load(f)


# Tests
# .............................................................................

@mock.patch('iga.ror.ror_data', new=mocked_ror_data)
def test_name_from_ror():
    assert name_from_ror('https://ror.org/05dxps055') == 'California Institute of Technology'
    assert name_from_ror('05dxps055') == 'California Institute of Technology'
    assert name_from_ror('https://ror.org/03ybx0x41') == 'University of Basel'



# =============================================================================
# @file    test_name_utils.py
# @brief   Py.test cases for parts of name_utils.py
# @created 2023-03-02
# @license Please see the file named LICENSE in the project directory
# @website https://github.com/caltechlibrary/iga
# =============================================================================

from iga.name_utils import (
    _cleaned_name,
    _plain_word,
    _first_letters_upcased,
    contains_cjk,
    flattened_name,
)

RAW_NAMES = [
    ('偏右', ''),
    ('王爵nice', 'nice'),
    ('勾三股四', ''),
    ('TZ | 天猪', 'TZ'),
    ('Mu-An ✌️ Chiou', 'Mu-An Chiou'),
]


def test_contains_cjk():
    assert contains_cjk('偏右')
    assert contains_cjk('王爵nice')
    assert not contains_cjk('test')
    assert not contains_cjk('éä')


def test_flattened_name():
    assert flattened_name('foo bar') == 'foo bar'
    assert flattened_name(['foo', 'bar']) == 'foo bar'
    assert flattened_name(['Foo J.', 'Bar']) == 'Foo J. Bar'
    assert flattened_name(['Foo', 'J.', 'Bar']) == 'Foo J. Bar'


def test_cleaned_name():
    for original, cleaned in RAW_NAMES:
        assert _cleaned_name(original) == cleaned


def test_plain_word():
    assert _plain_word('foo')
    assert _plain_word('FOO')
    assert _plain_word('Foo')
    assert not _plain_word('foo bar')
    assert not _plain_word('foo1')


def test_upcase_first_letter():
    assert _first_letters_upcased('aBC dEF') == 'ABC DEF'
    assert _first_letters_upcased('Abc Def') == 'Abc Def'
    assert _first_letters_upcased('abc def') == 'Abc Def'
    assert _first_letters_upcased('abc DEF') == 'Abc DEF'
    assert _first_letters_upcased('abc dEF') == 'Abc DEF'
    assert _first_letters_upcased('a') == 'A'
    assert _first_letters_upcased('A') == 'A'



# =============================================================================
# @file    test_cli.py
# @brief   Py.test cases for module command-line interface
# @created 2022-12-08
# @license Please see the file named LICENSE in the project directory
# @website https://github.com/caltechlibrary/iga
# =============================================================================

import click
import click.testing
import json5
import os
from   os import path
from   sidetrack import log
from   unittest import mock

from   iga.exit_codes import ExitCode
from iga.github import (
    GitHubRepo,
    GitHubAccount,
    GitHubRelease,
)


# Mocks
# .............................................................................

here      = path.dirname(path.abspath(__file__))
repo_dir  = path.join(here, 'data/fake-example/')
orcid_dir = path.join(here, 'data/orcid-examples/')


def mocked_github_account(account_name):
    log(f'returing mocked GitHubAccount for {account_name}')
    with open(path.join(repo_dir, 'account.json'), 'r') as f:
        return GitHubAccount(json5.loads(f.read()))


def mocked_github_repo(account_name, repo_name):
    log(f'returing mocked GitHubRepo for {repo_name}')
    with open(path.join(repo_dir, 'repo.json'), 'r') as f:
        repo = GitHubRepo(json5.loads(f.read()))
        repo._files = mocked_github_repo_filenames(repo_name, 'faketag')
        return repo


def mocked_github_release(account_name, repo_name, tag_name):
    log(f'returing mocked GitHubRelease for {tag_name}')
    with open(path.join(repo_dir, 'release.json'), 'r') as f:
        return GitHubRelease(json5.loads(f.read()))


def mocked_github_repo_filenames(repo, tag_name):
    log('returing mocked filenames list')
    with open(path.join(repo_dir, 'filenames.json'), 'r') as f:
        return json5.loads(f.read())


def mocked_github_repo_file(repo, tag_name, filename):
    log(f'returing mocked file contents for {filename}')
    with open(path.join(repo_dir, filename), 'r') as f:
        return f.read()


def mocked_github_repo_contributors(repo):
    log('returing mocked contributors')
    return []


def mocked_github_repo_languages(repo):
    log('returing mocked languages')
    return ['fakelanguage1', 'fakelanguage2']


def mocked_orcid_data(orcid):
    log(f'returing mocked orcid data for {orcid}')
    orcid_filename = orcid + '.json'
    with open(path.join(orcid_dir, orcid_filename), 'r') as f:
        return json5.load(f)


# Tests
# .............................................................................

@mock.patch.dict(os.environ, {}, clear=True)
@mock.patch('iga.github.github_repo_file', new=mocked_github_repo_file)
@mock.patch('iga.github.github_repo_filenames', new=mocked_github_repo_filenames)
@mock.patch('iga.github.github_repo_languages', new=mocked_github_repo_languages)
@mock.patch('iga.github.github_repo_contributors', new=mocked_github_repo_contributors)
@mock.patch('iga.github.github_account', new=mocked_github_account)
@mock.patch('iga.github.github_repo', new=mocked_github_repo)
@mock.patch('iga.github.github_release', new=mocked_github_release)
@mock.patch('iga.orcid.orcid_data', new=mocked_orcid_data)
def test_environment_vars_from_options(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    args = ['--invenio-server', 'https://data.caltechlibrary.dev',
            '--invenio-token', 'itoken',
            '--github-token', 'gtoken',
            '--save-metadata', '/tmp/fake.json',
            'https://github.com/fakeaccount/fakerepo/releases/tag/fakerelease']
    result = runner.invoke(cli, args)
    assert result.exit_code == int(ExitCode.success)
    assert 'INVENIO_SERVER' in os.environ
    assert os.environ['INVENIO_SERVER'] == 'https://data.caltechlibrary.dev'
    assert 'INVENIO_TOKEN' in os.environ
    assert os.environ['INVENIO_TOKEN'] == 'itoken'
    assert 'GITHUB_TOKEN' in os.environ
    assert os.environ['GITHUB_TOKEN'] == 'gtoken'


def test_no_args(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    result = runner.invoke(cli)
    assert 'Usage' in result.output
    assert result.exit_code == int(ExitCode.success)


def test_unknown_arg(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    result = runner.invoke(cli, ['--foo'])
    assert 'No such option' in result.output
    assert result.exit_code == int(ExitCode.bad_arg)


def test_help_flag(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    result = runner.invoke(cli, ['--help'])
    assert 'Usage' in result.output
    assert result.exit_code == int(ExitCode.success)
    result = runner.invoke(cli, ['-h'])
    assert 'Usage' in result.output
    assert result.exit_code == int(ExitCode.success)


# Currently broken.
# def test_help_arg(monkeypatch):
#     from iga.cli import cli
#     runner = click.testing.CliRunner()
#     result = runner.invoke(cli, ['help'])
#     assert result.exit_code == int(ExitCode.success)


def test_mode(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    result = runner.invoke(cli, ['--mode'])
    assert 'requires an argument' in result.output
    assert result.exit_code == int(ExitCode.bad_arg)


def test_version(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    result = runner.invoke(cli, ['--version'])
    assert 'version' in result.output
    assert result.exit_code == int(ExitCode.success)


def test_incomplete_file_arg(capsys):
    from iga.cli import cli
    runner = click.testing.CliRunner()
    result = runner.invoke(cli, ['-f'])
    assert 'requires an argument' in result.output
    assert result.exit_code == int(ExitCode.bad_arg)



'''
data_utils.py: miscellaneous data-handling utilities.

This file is part of https://github.com/caltechlibrary/iga/.

Copyright (c) 2023 by the California Institute of Technology.  This code
is open-source software released under a BSD-type license.  Please see the
file "LICENSE" for more information.
'''

from typing import Generator, Iterator
from url_normalize import url_normalize


def deduplicated(lst):
    '''Return a copy of lst from which duplicate items have been removed.'''
    if not lst:
        return []
    elif isinstance(lst, Generator):
        return deduplicated(list(lst))
    elif isinstance(lst, Iterator):
        return deduplicated([x for x in lst])
    elif not isinstance(lst, list):
        return lst
    elif isinstance(lst[0], dict):
        # Python dicts are not hashable, so comparing them is difficult. One
        # approach would be to use a set of frozensets to help uniquefy
        # values, but frozenset alone doesn't handle nested dictionaries.
        # The following simple solution is O(n^2) -- bad for big sets, but
        # our uses in IGA are small, so this is sufficient.
        deduplicated_list = []
        seen = []
        for item in lst:
            if item not in seen:
                seen.append(item)
                deduplicated_list.append(item)
        return deduplicated_list
    else:
        from commonpy.data_utils import unique
        return unique(lst)


def listified(thing):
    '''Return a list made out of thing.

    If thing is a single object, then this returns [thing].
    If list is a list, Iterator or Generator, this simply returns thing.
    '''
    if not thing:
        return []
    return thing if isinstance(thing, (list, Iterator, Generator)) else [thing]


def normalized_url(url):
    '''Return url but with some transformations to make it consistent.'''
    url = url.replace('https://git+https/github.com', 'https://github.com')
    url = url.replace('https://git@github.com:', 'https://github.com/')
    url = url.replace('git+https://github.com', 'https://github.com')
    url = url.replace('git+ssh://git@github.com:', 'https://github.com/')
    url = url.replace('git@github.com:', 'https://github.com/')
    url = url.replace('git://github.com/', 'https://github.com/')
    url = url.split('#')[0]
    url = url.removesuffix('.git')
    url = url_normalize(url)
    return url


def similar_urls(url1, url2):
    '''Return True if url1 is believed to be the same URL as url2.'''
    # I found things like urllib's urlparse and w3lib's canonicalize_url
    # unhelpful for this purpose. For example, for our cases we want to
    # assume https://foo.org/bar/ is the same as https://foo.org/bar, even
    # though by strict URL rules they're not. In the end I ended up doing this:
    return (
        url1 == url2
        or url1.removesuffix('/') == url2
        or url1 == url2.removesuffix('/')
        or url1.replace('http://', 'https://') == url2
        or url1.replace('https://', 'http://') == url2
        or url1.replace('http://', 'https://').removesuffix('/') == url2
        or url1.replace('http://', 'https://') == url2.removesuffix('/')
        or url1.replace('https://', 'http://').removesuffix('/') == url2
        or url1.replace('https://', 'http://') == url2.removesuffix('/')
    )



import json5
from   os import path
from   sidetrack import log
from   unittest import mock

from iga.orcid import name_from_orcid


# Mocks
# .............................................................................

here      = path.dirname(path.abspath(__file__))
orcid_dir = path.join(here, 'data/orcid-examples/')


def mocked_orcid_data(orcid):
    log(f'returing mocked ORCID data for {orcid}')
    orcid_filename = orcid + '.json'
    with open(path.join(orcid_dir, orcid_filename), 'r') as f:
        return json5.load(f)


# Tests
# .............................................................................

@mock.patch('iga.orcid.orcid_data', new=mocked_orcid_data)
def test_name_from_orcid():
    assert name_from_orcid('0000-0001-9105-5960') == ('Michael', 'Hucka')
    assert name_from_orcid('https://orcid.org/0000-0003-0900-6903') == ('R. S.', 'Doiel')
    assert name_from_orcid('https://orcid.org/0000-0002-8876-7606') == ('Neil P.', 'Chue Hong')
    assert name_from_orcid('https://orcid.org/0000-0001-6151-2200') == ('', '')
